@@ -1,15 +1,20 @@
-import secrets
-import string
+import sys
 from datetime import date, datetime, timezone
 from enum import Enum
-from time import sleep
-from typing import Mapping, Union
+from typing import Mapping
 
-import orjson
+from loguru import logger
 from pydantic import BaseModel
 
-from ._jupyter_communicate import notebook_path
-from ._logger import logger
+from .dev._dependency import infer_dependencies
+from .dev._initialize import initialize_metadata
+from .dev._jupyter_communicate import notebook_path
+from .dev._jupyter_lab_commands import _restart_notebook, _save_notebook
+from .dev._notebook import read_notebook, write_notebook
+
+_filepath = None
+_env = None
+_time_run = None
 
 
 def table_html(rows: list):
@@ -30,27 +35,6 @@ def display_html(html: str):
     display(HTML(html))
 
 
-def nbproject_id():  # rename to nbproject_id also in metadata slot?
-    """An 8-byte ID encoded as a 12-character base62 string."""
-    # https://github.com/laminlabs/notes/blob/main/2022-04-04-human-friendly-ids.ipynb
-    base62 = string.digits + string.ascii_letters.swapcase()
-    id = "".join(secrets.choice(base62) for i in range(12))
-    return id
-
-
-# schema within the metadata section
-class JSONSchema(BaseModel):
-    nbproject_id: str  # a full 32 digit uid4.hex string
-    nbproject_time_init: datetime
-
-
-# user visible name & type configuration
-class UserSchema(BaseModel):
-    id: Union[str, int] = nbproject_id  # type: ignore
-    time_init: Union[date, datetime]  # type: ignore
-    time_run: Union[date, datetime]  # not part of the ipynb metadata section
-
-
 # display configuration
 class DisplayConf(BaseModel):
     time_init: Enum("choice", ["date", "datetime"]) = "datetime"  # type: ignore # noqa
@@ -58,7 +42,7 @@ class DisplayConf(BaseModel):
 
 
 # displays fields within the ipynb metadata section and on-the-fly computed
-class Display:
+class DisplayMeta:
     def __init__(self, nb_metadata=None):
         self.metadata = nb_metadata
         self.conf = DisplayConf()
@@ -89,18 +73,41 @@ class Display:
                 "%Y-%m-%d %H:%M"
             )  # probably something more reduced is better
 
+    def version(self):
+        if "version" in self.metadata["nbproject"]:
+            return self.metadata["nbproject"]["version"]
+        else:
+            return "draft"  # for backward-compat right now
+
     def dependency(self, deps: Mapping = None):
         if deps is None and "dependency" in self.metadata["nbproject"]:
             deps = self.metadata["nbproject"]["dependency"]
-        if deps is not None:
-            deps_list = [
-                pkg + f"=={ver}" if ver != "" else pkg for pkg, ver in deps.items()
-            ]
-            deps_display = None if deps_list == [] else " ".join(deps_list)
-        return deps_display
+
+        if deps is None:
+            return None
+
+        deps_list = []
+        for pkg, ver in deps.items():
+            if ver != "":
+                deps_list.append(pkg + f"=={ver}")
+            else:
+                deps_list.append(pkg)
+
+        if deps_list == []:
+            return None
+        else:
+            deps_list.sort()
+            return deps_list
 
 
 class Header:
+    """Metadata header class.
+
+    An object of this class displays nbproject metadata fields
+    for the current notebook on initialization. If the notebook doesn't have
+    nbproject metadata, it will be initialized and written to the notebook.
+    """
+
     def __init__(self, filepath=None, env=None):
         filepath_env = filepath, env
 
@@ -111,82 +118,101 @@ class Header:
                     "can't infer the name of the current notebook, "
                     "you are probably not inside a jupyter notebook"
                 )
+            filepath = filepath_env[0]
+
+        # without this, we have ugly timestamps
+        logger.configure(
+            handlers=[
+                dict(
+                    sink=sys.stdout,
+                    format="{message}",
+                ),
+            ],
+        )
 
-        filepath = filepath_env[0]
         if env is None:
             env = filepath_env[1]
+        # This is a quirk we run into when passing filepath manually!
+        # We just assume jupyter lab as an environment for now
+        if env is None:
+            env = "lab"
+            logger.info("... assuming editor is Jupyter Lab")
 
         try:
-            with open(filepath, "rb") as f:
-                nb = orjson.loads(f.read())
+            nb = read_notebook(filepath)
         except FileNotFoundError:
             raise RuntimeError(
                 "try passing the filepath manually to nbproject.Header()"
             )
         # initialize
-        from ._dependency import notebook_deps
-
-        if "nbproject" not in nb["metadata"]:
+        if "nbproject" not in nb.metadata:
             logger.info(
                 "To initialize nbproject for this notebook:\n* In Jupyter Lab: hit"
                 " restart when asked!"
             )
 
             if env == "lab":
-                from ipylab import JupyterFrontEnd
+                _save_notebook()
+                nb = read_notebook(filepath)
 
-                app = JupyterFrontEnd()
-                # ensure that all user edits are saved before the
-                # notebook will be loaded by the backend
-                app.commands.execute("docmanager:save")
-                # it's important that the frontend is done saving
-                # before the backend loads
-                sleep(1)
-
-            # now load the notebook with the backend
-            with open(filepath, "rb") as f:
-                nb = orjson.loads(f.read())
             # write metadata from the backend
-            nb["metadata"]["nbproject"] = {}
-            nb["metadata"]["nbproject"]["id"] = nbproject_id()
-            nb["metadata"]["nbproject"]["time_init"] = datetime.now(
-                timezone.utc
-            ).isoformat()
-            nb["metadata"]["nbproject"]["dependency"] = notebook_deps(
-                nb, pin_versions=True
-            )
+            nb.metadata["nbproject"] = initialize_metadata(nb).dict()
 
             # write the file from the backend
-            with open(filepath, "wb") as f:
-                f.write(orjson.dumps(nb))
+            write_notebook(nb, filepath)
 
             if env == "lab":
                 # reload the notebook with metadata by the frontend
                 # otherwise Jupyter lab notices the mismatch
                 # and shows a confusing dialogue
-                app.commands.execute("docmanager:reload")
-                # restart and re-execute `from nbproject import header`
-                app.commands.execute("notebook:restart-and-run-to-selected")
+                _restart_notebook()
+
         # read from ipynb metadata and add on-the-fly computed metadata
         else:
-
             # display metadata
-            display_ = Display(nb["metadata"])
+            dm = DisplayMeta(nb.metadata)
 
-            time_run = display_.time_run(datetime.now(timezone.utc))
+            time_run = datetime.now(timezone.utc)
+
+            # make time_run available through API
+            global _time_run
+            _time_run = time_run
 
             table = []
-            table.append(["id", display_.id()])
-            table.append(["time_init", display_.time_init()])
-            table.append(["time_run", time_run])
-            deps = notebook_deps(nb, pin_versions=True)
-            deps_display = display_.dependency(deps)
-            if deps_display is not None:
-                table.append(["dependency", deps_display])
+            table.append(["id", dm.id()])
+            table.append(["time_init", dm.time_init()])
+            table.append(["time_run", dm.time_run(time_run)])
+            table.append(["version", dm.version()])
+
+            dep_store = dm.dependency()
+            add_pkgs = None
+
+            if dep_store is not None:
+                # only display stored dependencies for published notebooks
+                # for draft notebooks, they have little meaning
+                if (
+                    "version" in nb.metadata["nbproject"]
+                    and nb.metadata["nbproject"]["version"] != "draft"  # noqa
+                ):
+                    table.append(["dependency_store", " ".join(dep_store)])
+                add_pkgs = [pkg.partition("==")[0] for pkg in dep_store]
+
+            dep_live = dm.dependency(
+                infer_dependencies(nb, add_pkgs, pin_versions=True)
+            )
+            suffix = ""
+            if (
+                "version" in nb.metadata["nbproject"]
+                and nb.metadata["nbproject"]["version"] != "draft"  # noqa
+            ):
+                suffix = "_live"
+            table.append([f"dependency{suffix}", " ".join(dep_live)])
 
             display_html(table_html(table))
 
         # make filepath available through API
-        import nbproject._meta
+        global _filepath
+        global _env
 
-        nbproject._meta._filepath = filepath
+        _filepath = filepath
+        _env = env
