@@ -6,6 +6,11 @@ from ._logger import logger
 from .dev._initialize import MetaStore
 from .dev._integrity import check_integrity
 from .dev._jupyter_communicate import notebook_path
+from .dev._jupyter_lab_commands import (
+    _reload_shutdown,
+    _restart_notebook,
+    _save_notebook,
+)
 from .dev._notebook import Notebook, read_notebook, write_notebook
 
 
@@ -36,8 +41,14 @@ class MetaLive:
     on access from the notebook's content.
     """
 
-    def __init__(self, nb_path: Union[str, Path], time_run: Optional[datetime] = None):
+    def __init__(
+        self,
+        nb_path: Union[str, Path],
+        time_run: Optional[datetime] = None,
+        env: Optional[str] = None,
+    ):
         self._nb_path = nb_path
+        self._env = env
         self._time_run = time_run
 
     @property
@@ -55,15 +66,22 @@ class MetaLive:
         from .dev._dependency import infer_dependencies
 
         nb = read_notebook(self._nb_path)
-        return infer_dependencies(nb, pin_versions=True)
+        add_pkgs = None
+        if "nbproject" in nb.metadata and "dependency" in nb.metadata["nbproject"]:
+            add_pkgs = nb.metadata["nbproject"]["dependency"].keys()
+        return infer_dependencies(nb, add_pkgs, pin_versions=True)
 
     @property
     def integrity(self):
-        """Check integrity of the notebook.
+        """Compute integrity of the notebook.
 
-        The notebook should be saved before accessing this attribute.
+        Returns those cell transitions that violate execution at increments of 1
+        as a list of tuples.
         """
-        logger.info("Save the notebook before running the integrity check.")
+        if self._env == "lab":
+            _save_notebook()
+        else:
+            logger.info("Save the notebook before running the integrity check.")
         nb = read_notebook(self._nb_path)
         return check_integrity(nb, ignore_code=".live.integrity")
 
@@ -96,13 +114,20 @@ class Meta:
     derived from the notebook's content.
     """
 
-    def __init__(self, filepath, time_run):
+    def __init__(self, filepath, time_run, env):
+        filepath_env = filepath, env
+
         if filepath is None:
-            filepath = notebook_path()
+            filepath_env = notebook_path(return_env=True)
+            filepath = filepath_env[0]
+
+        if env is None:
+            env = filepath_env[1]
 
         self._filepath = filepath
+        self._env = env
 
-        self._live = MetaLive(filepath, time_run)
+        self._live = MetaLive(filepath, time_run, env)
 
         nb_meta = read_notebook(filepath).metadata
         if "nbproject" in nb_meta:
@@ -120,18 +145,60 @@ class Meta:
         """Contains execution info and properties of the notebook content."""
         return self._live
 
-    def write(self):
-        """Write nbproject metadata in `.store` to the current file.
+    def write(self, restart=True):
+        """Write metadata in `.store` to file and shutdown notebook kernel.
 
         You can edit the nbproject metadata of the current notebook
         by changing `.store` fields and then using this function
         to write the changes to the file. Save the notebook before writing.
         """
-        logger.info("Restart the notebook.")
+        if self._env == "lab":
+            _save_notebook()
+
         nb = read_notebook(self._filepath)
         nb.metadata["nbproject"] = self.store.dict()
+        # also update displayed version number
+        # this is ugly right now but important
+        version_icell = None
+        version_ioutput = None
+        for icell, cell in enumerate(nb.cells):
+            if "outputs" in cell:
+                for ioutput, output in enumerate(cell["outputs"]):
+                    if "data" in output:
+                        if "text/html" in output["data"]:
+                            if "<b>version</b>" in output["data"]["text/html"][0]:
+                                version_icell = icell
+                                version_ioutput = ioutput
+                                break
+        if version_icell is not None:
+            nb.cells[version_icell]["outputs"][version_ioutput]["data"]["text/html"][
+                0
+            ] = nb.cells[version_icell]["outputs"][version_ioutput]["data"][
+                "text/html"
+            ][
+                0
+            ].replace(
+                "<b>version</b></td><td style='text-align: left;'>draft</td>",
+                "<b>version</b></td><td style='text-align:"
+                f" left;'>{self.store.version}</td>",  # noqa
+            )
+        else:
+            logger.warning("... could not update displayed version")
         write_notebook(nb, self._filepath)
 
+        if self._env == "lab":
+            if restart:
+                _restart_notebook()
+            else:
+                logger.info("... reload notebook from disk & shutdown kernel")
+                _reload_shutdown()
+        else:
+            logger.info(
+                "Shut down kernel as file changed on disk. Reload and restart the"
+                " notebook if you want to continue."
+            )
+            # sys.exit(0)  # makes CI fail, need to think of a decent way of exiting
+
     def __repr__(self):
         return (
             "Metadata object with .live and .store metadata fields:\n"
@@ -141,7 +208,7 @@ class Meta:
 
 
 def _load_meta():
-    from ._header import _filepath, _time_run
+    from ._header import _env, _filepath, _time_run
 
-    meta = Meta(_filepath, _time_run)
+    meta = Meta(_filepath, _time_run, _env)
     return meta
